@@ -1,4 +1,6 @@
 import dask  # noqa
+import numpy as np
+import xarray as xr
 import xclim as xc
 from xclim.core.calendar import percentile_doy
 
@@ -21,9 +23,14 @@ def _get_da(dictionary, var):
 def _get_percentile(da, perc, base_period_time_range):
     tslice = slice(base_period_time_range[0], base_period_time_range[1])
     base_period = da.sel(time=tslice)
-    with dask.config.set(**{"array.slicing.split_large_chunks": True}):
+    with dask.config.set(**{"array.slicing.split_large_chunks": False}):
         per_doy = percentile_doy(base_period, per=perc)
-    return per_doy.sel(percentiles=perc)
+        per_doy_comp = per_doy.compute()
+    return per_doy_comp.sel(percentiles=perc)
+
+
+def _convert_snow_mm_day(da):
+    return da / 312 * 1000
 
 
 BASE_PERIOD = ["1971-01-01", "2000-12-31"]
@@ -58,11 +65,12 @@ class CD:
             perc=25,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.cold_and_dry_days(
-            tas_per=percentile_tas,
-            pr_per=percentile_pr,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.cold_and_dry_days(
+                tas_per=percentile_tas,
+                pr_per=percentile_pr,
+                **params,
+            )
 
 
 class CDD:
@@ -151,11 +159,12 @@ class CSDI:
             perc=10,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.cold_spell_duration_index(
-            tasmin_per=percentile,
-            window=6,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.cold_spell_duration_index(
+                tasmin_per=percentile,
+                window=6,
+                **params,
+            )
 
 
 class CSU:
@@ -211,11 +220,12 @@ class CW:
             perc=75,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.cold_and_wet_days(
-            tas_per=percentile_tas,
-            pr_per=percentile_pr,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.cold_and_wet_days(
+                tas_per=percentile_tas,
+                pr_per=percentile_pr,
+                **params,
+            )
 
 
 class CWD:
@@ -573,10 +583,11 @@ class RDYYp:
             perc=perc,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.days_over_precip_doy_thresh(
-            pr_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.days_over_precip_doy_thresh(
+                pr_per=percentile,
+                **params,
+            )
 
 
 class RYYmm:
@@ -676,10 +687,11 @@ class RYYpTOT:
             perc=perc,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.fraction_over_precip_thresh(
-            pr_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.fraction_over_precip_thresh(
+                pr_per=percentile,
+                **params,
+            )
 
 
 class SDII:
@@ -794,10 +806,11 @@ class TG10p:
             perc=10,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.tg10p(
-            tas_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.tg10p(
+                tas_per=percentile,
+                **params,
+            )
 
 
 class TG90p:
@@ -824,10 +837,11 @@ class TG90p:
             perc=90,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.tg90p(
-            tas_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.tg90p(
+                tas_per=percentile,
+                **params,
+            )
 
 
 class TR:
@@ -898,10 +912,11 @@ class TX10p:
             perc=10,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.tx10p(
-            tasmax_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.tx10p(
+                tasmax_per=percentile,
+                **params,
+            )
 
 
 class TX90p:
@@ -928,10 +943,11 @@ class TX90p:
             perc=90,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.tx90p(
-            tasmax_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.tx90p(
+                tasmax_per=percentile,
+                **params,
+            )
 
 
 class TXn:
@@ -1015,10 +1031,11 @@ class TN10p:
             perc=10,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.tn10p(
-            tasmin_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.tn10p(
+                tasmin_per=percentile,
+                **params,
+            )
 
 
 class TN90p:
@@ -1045,10 +1062,11 @@ class TN90p:
             perc=90,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.tn90p(
-            tasmin_per=percentile,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.tn90p(
+                tasmin_per=percentile,
+                **params,
+            )
 
 
 class TNn:
@@ -1118,11 +1136,12 @@ class WD:
             perc=25,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.warm_and_dry_days(
-            tas_per=percentile_tas,
-            pr_per=percentile_pr,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.warm_and_dry_days(
+                tas_per=percentile_tas,
+                pr_per=percentile_pr,
+                **params,
+            )
 
 
 class WSDI:
@@ -1149,11 +1168,12 @@ class WSDI:
             perc=90,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.warm_spell_duration_index(
-            tasmax_per=percentile,
-            window=6,
-            **params,
-        )
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.warm_spell_duration_index(
+                tasmax_per=percentile,
+                window=6,
+                **params,
+            )
 
 
 class WW:
@@ -1185,8 +1205,220 @@ class WW:
             perc=75,
             base_period_time_range=base_period_time_range,
         )
-        return xc.atmos.warm_and_wet_days(
-            tas_per=percentile_tas,
-            pr_per=percentile_pr,
+        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+            return xc.atmos.warm_and_wet_days(
+                tas_per=percentile_tas,
+                pr_per=percentile_pr,
+                **params,
+            )
+
+
+class CSf:
+    """Number of cold spells (tas)."""
+
+    thresh = -10
+    window = 3
+
+    def compute(thresh=thresh, window=window, **params):
+        """Calculate number of cold spells.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#cold_spell_frequency
+
+        Returns
+        -------
+        Number of cold spells of at least {window} consecutive days
+        with temperature below {thresh}.
+        """
+        thresh = _thresh_string(thresh, "degC")
+        return xc.atmos.cold_spell_frequency(
+            thresh=thresh,
+            window=window,
+            **params,
+        )
+
+
+class HSf:
+    """Number of hot spells (tasmax)."""
+
+    thresh = 35
+    window = 3
+
+    def compute(thresh=thresh, window=window, **params):
+        """Calculate number of hot spells.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#hot_spell_frequency
+
+        Returns
+        -------
+        Number of hot spells of at least {window} consecutive days
+        with maximum temperature above {thresh}.
+        """
+        thresh = _thresh_string(thresh, "degC")
+        return xc.atmos.hot_spell_frequency(
+            thresh_tasmax=thresh,
+            window=window,
+            **params,
+        )
+
+
+class HSx:
+    """Maximum lenght of hot spells (tasmax)."""
+
+    thresh = 35
+    window = 1
+
+    def compute(thresh=thresh, window=window, **params):
+        """Calculate maximum lenght of hot spells.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#hot_spell_max_length
+
+        Returns
+        -------
+        Maximum length of hot spells of at least {window} consecutive days
+        with maximum temperature above {thresh}.
+        """
+        thresh = _thresh_string(thresh, "degC")
+        return xc.atmos.hot_spell_max_length(
+            thresh_tasmax=thresh,
+            window=window,
+            **params,
+        )
+
+
+class SD:
+    """Number of snow days."""
+
+    thresh = 1
+
+    def compute(thresh=thresh, **params):
+        """Calculate number of snow days.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#days_with_snow
+
+        Returns
+        -------
+        Number of days with solid precipitation flux above {thresh} threshold.
+        """
+        thresh = _thresh_string(thresh, "mm/day")
+        da = _get_da(params, "prsn")
+        prsn = _convert_snow_mm_day(da)
+        prsn.attrs["units"] = "kg m-2 s-1"
+        if "ds" in params.keys():
+            del params["ds"]
+        params["prsn"] = prsn
+        return xc.atmos.days_with_snow(
+            low=thresh,
+            **params,
+        )
+
+
+class SCD:
+    """Snow cover duration."""
+
+    thresh = 1
+
+    def compute(thresh=thresh, **params):
+        """Calculate snow cover duration.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#snow_cover_duration
+
+        Returns
+        -------
+        Number of days with snow cover above {thresh} threshold.
+        """
+        thresh = _thresh_string(thresh, "cm")
+        return xc.land.snow_cover_duration(
+            thresh=thresh,
+            **params,
+        )
+
+
+class Sint:
+    """Snowfall intensity."""
+
+    def compute(**params):
+        """Calculate snowfall intensity.
+        Parameters
+        ----------
+
+        Returns
+        -------
+        Mean daily snowfall during days with snowfall > 1mm/day
+        """
+
+        da = _get_da(params, "prsn")
+        prsn = _convert_snow_mm_day(da) * 86400
+        if "ds" in params.keys():
+            del params["ds"]
+        masked = xr.where(prsn > 1, prsn, np.nan)
+        mean = masked.resample(time=params["freq"]).mean(dim="time")
+        return xr.where(mean == np.nan, 0, mean)
+
+
+class Sfreq:
+    """Snowfall frequency."""
+
+    def compute(**params):
+        """Calculate snowfall frequency.
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#days_with_snow
+
+        Returns
+        -------
+        Percentage of days with snowfall > 1mm/day.
+        """
+        thresh = _thresh_string(1, "mm/day")
+        da = _get_da(params, "prsn")
+        prsn = _convert_snow_mm_day(da)
+        prsn.attrs["units"] = "kg m-2 s-1"
+        if "ds" in params.keys():
+            del params["ds"]
+        params["prsn"] = prsn
+        sd = xc.atmos.days_with_snow(
+            low=thresh,
+            **params,
+        )
+        ndays = da.resample(time=params["freq"]).count(dim="time")
+        return sd / ndays * 100
+
+
+class UTCI:
+    """Universal thermal climate index."""
+
+    stat = "average"
+    mask_invalid = True
+
+    def compute(stat=stat, mask_invalid=mask_invalid, **params):
+        """Calculate universal thermal climate index.
+
+        Parameters
+        ----------
+        For input parameters see:
+            https://xclim.readthedocs.io/en/stable/indicators_api.html#universal_thermal_cliamte_index
+
+        Returns
+        -------
+        Universal Thermal Climate Index.
+        """
+        return xc.atmos.universal_thermal_climate_index(
+            stat=stat,
+            mask_invalid=mask_invalid,
             **params,
         )
