@@ -6,8 +6,13 @@ import pyhomogenize as pyh
 import xarray as xr
 
 from . import _indices as indices
-from ._consts import _freq, _tfreq
-from ._utils import check_existance, kwargs_to_self, object_attrs_to_self
+from ._consts import _bfreq, _freq, _tfreq
+from ._utils import (
+    check_existance,
+    get_time_bounds,
+    kwargs_to_self,
+    object_attrs_to_self,
+)
 
 
 class Processing:
@@ -75,7 +80,6 @@ class Processing:
 
     def _get_replacement(self, obj, numb_name):
         replacement = {}
-        repl_value = ""
         for attr in dir(obj):
             if attr[0] == "_":
                 continue
@@ -87,42 +91,40 @@ class Processing:
                 replacement[attr] = numb_name
             else:
                 replacement[attr] = getattr(obj, attr)
-            if repl_value == "":
-                if isinstance(replacement[attr], list):
-                    continue
-                repl_value = replacement[attr]
-                if isinstance(repl_value, str) and len(repl_value) > 0:
-                    if repl_value[0] == "0":
-                        repl_value = float(
-                            "{}.{}".format(
-                                repl_value[0],
-                                repl_value[1:],
-                            )
+            if isinstance(replacement[attr], list):
+                continue
+            if isinstance(replacement[attr], str):
+                if replacement[attr][0] == "0":
+                    replacement[attr] = float(
+                        "{}.{}".format(
+                            replacement[attr][0],
+                            replacement[attr][1:],
                         )
-                    else:
-                        repl_value = int(repl_value)
-                replacement[attr] = repl_value
-                repl_value = str(repl_value)
-        return replacement, repl_value
+                    )
+                else:
+                    replacement[attr] = int(replacement[attr])
+        return replacement
 
     def _get_idx_name_and_repl(self):
         numb_name, idx_object = self._get_numb_name_and_idx_object()
+        defaults = [attr for attr in dir(idx_object)]
         object_attrs_to_self(idx_object, self, overwrite=False)
-        self.replacement, self.repl_value = self._get_replacement(
+        self.replacement = self._get_replacement(
             idx_object,
             numb_name,
         )
-        if not self.repl_value:
-            pass
-        elif "YY" in self.CIname:
-            self.CIname = self.CIname.replace("YY", self.repl_value)
-        elif numb_name:
-            self.CIname = self.CIname = self.CIname.replace(
-                numb_name,
-                self.repl_value,
-            )
-        elif self.repl_value not in self.CIname:
-            self.CIname = "{}{}".format(self.CIname, self.repl_value)
+        for k, v in self.replacement.items():
+            if k in defaults and k in self.kwargs.keys():
+                if "YY" in self.CIname:
+                    self.CIname = self.CIname.replace("YY", str(v))
+                elif numb_name:
+                    self.CIname = self.CIname = self.CIname.replace(
+                        numb_name,
+                        v,
+                    )
+                elif str(v) not in self.CIname:
+                    self.CIname = "{}{}".format(self.CIname, str(v))
+                break
 
     def _adjust_params_to_ci(self):
         params = {
@@ -142,26 +144,34 @@ class Processing:
             end=self.preproc.time.values[-1],
             frequency=_tfreq[self.freq],
         )
-        data_vars = {
-            k: self.preproc.data_vars[k]
-            for k in self.preproc.data_vars.keys()
-            if k not in self.var_name
-        }
-        data_vars[self.CIname] = array
+        dvars = self.preproc.data_vars
+        data_vars = {k: v for k, v in dvars.items() if k not in self.var_name}
         if "time_bnds" in data_vars.keys():
             del data_vars["time_bnds"]
-        if "time_bounds" in data_vars.keys():
+        elif "time_bounds" in data_vars.keys():
             del data_vars["time_bounds"]
-        idx_ds = xr.Dataset(data_vars=data_vars, attrs=self.preproc.attrs)
-        if len(idx_ds.time) > 1:
-            idx_ds = idx_ds.assign_coords(
-                {"time": date_range.to_datetimeindex},
-            )
-            idx_ds = idx_ds.cf.add_bounds("time")
-            idx_ds = idx_ds.reset_coords("time_bounds")
+        data_vars[self.CIname] = array
+        coords = {k: v for k, v in self.ds.coords.items() if "time" not in k}
+        idx_ds = xr.Dataset(
+            data_vars=data_vars,
+            coords=coords,
+            attrs=self.preproc.attrs,
+        )
+        idx_ds = idx_ds.assign_coords(
+            {"time": date_range},
+        )
+        idx_ds.time.encoding = self.ds.time.encoding
+        t_bounds = get_time_bounds(
+            self.preproc.time.values[0],
+            self.preproc.time.values[-1],
+            idx_ds.time,
+            l_freq=_bfreq[self.freq][0],
+            u_freq=_bfreq[self.freq][1],
+            td=_bfreq[self.freq][2],
+        )
+        idx_ds["time_bnds"] = t_bounds
+        idx_ds["time"].attrs["bounds"] = "time_bnds"
         for data_var in idx_ds.data_vars:
             data_var_repl = data_var.replace("bounds", "bnds")
             idx_ds = idx_ds.rename({data_var: data_var_repl})
-        idx_ds = idx_ds.assign_coords({"time": date_range})
-        idx_ds.time.encoding = self.ds.time.encoding
         return idx_ds
